﻿Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.IO

Namespace AutoDialer
    Public Class cVaxServerExCOM
        Inherits cVaxServerCOM

        Public Shared mVaxExCOM As cVaxServerExCOM

        Private m_nMusicAutoDialer As Integer = -1
        Private m_bIsLineAdded As Boolean = False
        Private m_bPlayWaveConnected As Boolean = False

        Private m_bDetectDTMF As Boolean = False
        Private m_bDetectAMD As Boolean = False
        Private m_bInitialized As Boolean = False

        Private m_objSessionList As cVaxCallSessionList = Nothing
        Private m_objMainForm As MainForm

        Public Const VAX_ACCOUNT_LINE_SIP As String = "LineSIP"
        Public Const VAX_ACCOUNT_QUEUE_SIP As String = "QueueInterView"

        Public Sub New(ByVal objMainForm As MainForm)

            mVaxExCOM = Me

            m_objMainForm = objMainForm
            ResetAll()

        End Sub

        Protected Overrides Sub Finalize()
        End Sub

        Private Sub ResetAll()
            m_objSessionList = New cVaxCallSessionList()
        End Sub

        Public Overloads Function Initialize(ByVal nListenPort As Integer) As Boolean
            UnInitialize()
            MyBase.SetLicenseKey("TRIAL-LICENSE-KEY")
            If Not MyBase.Initialize("") Then Return False

            If Not MyBase.OpenNetworkUDP("", nListenPort) Then
                UnInitialize()
                Return False
            End If

            MyBase.SetListenPortRangeRTP(20000, 40000)
            MyBase.AudioSessionLost(True, 20)
            MyBase.VideoCOMM(False)
            MyBase.AddQueue(VAX_ACCOUNT_QUEUE_SIP)
            MyBase.SetQueueProcessMode(VAX_ACCOUNT_QUEUE_SIP, VAX_QUEUE_PROCESS_MODE_RING_ALL, VAX_QUEUE_PROCESS_TYPE_FREE)

            For nCount As Integer = 4000 To 4080
                MyBase.AddUser(nCount.ToString(), nCount.ToString(), "32")
                MyBase.AddQueueAgent(VAX_ACCOUNT_QUEUE_SIP, nCount.ToString())
            Next

            m_bInitialized = True
            Return True
        End Function

        Public Overloads Sub UnInitialize()
            m_bInitialized = False
            UnLoadMusicAutoDialer()
            ResetAll()
            m_nMusicAutoDialer = -1
            m_bIsLineAdded = False
            MyBase.UnInitialize()
        End Sub

        Public Function IsInitialized() As Boolean
            Return m_bInitialized
        End Function

        Public Function RegisterAccountSIP(ByVal sDisplayName As String, ByVal sUserName As String, ByVal sAuthUser As String, ByVal sAuthPwd As String, ByVal sDomainRealm As String, ByVal sServerAddr As String, ByVal nServerPort As Integer) As Boolean
            If Not MyBase.AddLine(VAX_ACCOUNT_LINE_SIP, VAX_LINE_TYPE_UDP, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sServerAddr, nServerPort, "32") Then Return False
            If Not RegisterLine(VAX_ACCOUNT_LINE_SIP, 1800) Then Return False
            m_bIsLineAdded = True
            Return True
        End Function

        Public Sub UnRegisterAccountSIP()
            MyBase.UnRegisterLine(VAX_ACCOUNT_LINE_SIP)
            MyBase.RemoveLine(VAX_ACCOUNT_LINE_SIP)
            m_bIsLineAdded = False
        End Sub

        Public Function IsAddedAccountSIP() As Boolean
            Return m_bIsLineAdded
        End Function

        Public Sub DialCall(ByVal sDialNo As String, ByVal nPhoneNoId As UInt32)
            Dim nSessionId As Integer = CInt(DialCallSession("", "", sDialNo, VAX_ACCOUNT_LINE_SIP, 30))
            If nSessionId = -1 Then Return
            m_objSessionList.AddSession(nSessionId)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            objCallSession.m_sDialedNo = sDialNo
            objCallSession.m_sStatus = cDefine.PHONE_NO_PROCESSING
            objCallSession.m_nPhoneNoId = nPhoneNoId
            objCallSession.m_bDialNoCallType = True
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_PROCESSING, "", "")
        End Sub

        Public Sub CloseCall(ByVal nPhoneNoId As UInt32)
            Dim objCallSession As cVaxCallSession = m_objSessionList.FindPhoneNoSession(nPhoneNoId)
            If objCallSession Is Nothing Then Return
            CloseCallSession(objCallSession.m_nSessionId)
        End Sub

        Public Function LoadMusicAutoDialer(ByVal sWaveFile As String) As Boolean
            UnLoadMusicAutoDialer()
            If sWaveFile.Length = 0 Then Return False
            m_nMusicAutoDialer = MyBase.LoadWaveFile(sWaveFile)
            If m_nMusicAutoDialer = -1 Then Return False
            Return True
        End Function

        Public Sub UnLoadMusicAutoDialer()
            If m_nMusicAutoDialer = -1 Then Return
            MyBase.UnLoadWaveID(m_nMusicAutoDialer)
            m_nMusicAutoDialer = -1
        End Sub

        Public Sub PlayMusicAutoDialer(ByVal nSessionId As Integer)
            If m_nMusicAutoDialer = -1 Then Return
            MyBase.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, m_nMusicAutoDialer, True, 10)
        End Sub

        Public Sub PlayTextAutoDialer(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal sText As String)
            Dim aDataPCM As Byte() = Nothing
            Dim nDataSize As Integer = 0

            Dim objSpeech As cTextToSpeech = New cTextToSpeech()
            objSpeech.GenSpeech(sText, aDataPCM, nDataSize)

            MyBase.SendAudioPCM(nSessionId, nChannelId, aDataPCM, nDataSize)
        End Sub
        Public Sub EnableDetectDTMF(ByVal bDetectDTMF As Boolean)
            m_bDetectDTMF = bDetectDTMF
        End Sub

        Public Sub EnableDetectAMD(ByVal bDetectAMD As Boolean)
            m_bDetectAMD = bDetectAMD
        End Sub

        Public Sub SetPlayWaveConnected(ByVal bPlayWaveConnected As Boolean)
            m_bPlayWaveConnected = bPlayWaveConnected
        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////

        Protected Overrides Sub OnCallSessionCreated(ByVal nSessionId As Integer, ByVal nReasonCode As Integer)

            m_objSessionList.AddSession(nSessionId)

        End Sub

        Protected Overrides Sub OnCallSessionClosed(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nReasonCode As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            If objCallSession.IsConnected() Then
                objCallSession.EndCall()
                Dim sDuration As String = objCallSession.GetDuration()
                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, "", "", sDuration)
            End If

            objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode)
            m_objSessionList.RemoveSession(nSessionId)

        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        Protected Overrides Sub OnCallSessionAccessAudioPCM(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nAccessType As Integer, ByVal aDataPCM As Byte(), ByVal nSizePCM As Integer, ByVal nTypePCM As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM)

        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////

        Protected Overrides Sub OnCallSessionConnected(ByVal nSessionId As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            If objCallSession.m_bDialNoCallType = False Then Return

            objCallSession.OnCallSessionConnected(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE)

            If m_bPlayWaveConnected Then PlayMusicAutoDialer(nSessionId)
            If m_bDetectAMD Then MyBase.CallSessionDetectAMD(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, True, 5000, 300, 2)

            If m_bDetectDTMF Then
                MyBase.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, cVaxServerCOM.VAX_DTMF_TYPE_RFC2833, True, 2000)
                MyBase.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ONE, cVaxServerCOM.VAX_DTMF_TYPE_SIP_INFO, True, 2000)
            End If

            objCallSession.StartCall()
            objCallSession.SetConnected()

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_CONNECTED, "", "")

            If objCallSession.m_bDialNoCallType Then
                MyBase.AddCallSessionToQueue(VAX_ACCOUNT_QUEUE_SIP, "", "", nSessionId, 40)
            End If

        End Sub

        Protected Overrides Sub OnDetectedDigitDTMF(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal sDigitDTMF As String, ByVal nTypeDTMF As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, "", sDigitDTMF, "")

            Dim sMsgDTMF As String = "Digit Detected: [ " & sDigitDTMF & " ]"
            PlayTextAutoDialer(nSessionId, nChannelId, sMsgDTMF)

            GenStatusLog(sMsgDTMF)

        End Sub

        Protected Overrides Sub OnCallSessionFailed(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String, ByVal sContact As String)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            Dim sStatus As String
            sStatus = cDefine.PHONE_NO_FAILED & " : " & nStatusCode.ToString() & "-" & sReasonPhrase
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, sStatus, "", "")
        End Sub

        Protected Overrides Sub OnCallSessionTimeout(ByVal nSessionId As Integer, ByVal nChannelId As Integer)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_TIMEOUT, "", "")
        End Sub

        Protected Overrides Sub OnCallSessionCancelled(ByVal nSessionId As Integer)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.PHONE_NO_CANCELED, "", "")
        End Sub

        Protected Overrides Sub OnCallSessionDetectedAMD(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal bIsHuman As Boolean)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            If bIsHuman Then
                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.AMD_HUMAN_VOICE, "", "")
                Return
            End If

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sDialedNo, cDefine.AMD_MACHINE_VOICE, "", "")
        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        Protected Overrides Sub OnCallSessionMediaDetectStartedVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId)

        End Sub

        Protected Overrides Sub OnCallSessionMediaDetectCompletedVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal aDataPCM As Byte(), ByVal nSizePCM As Integer, ByVal nTypePCM As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM)

        End Sub

        Protected Overrides Sub OnCallSessionMediaDetectExpiredVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Protected Overrides Sub OnRegisterUser(ByVal sUserLogin As String, ByVal sDomain As String, ByVal sUserAgentName As String, ByVal sFromIP As String, ByVal nFromPort As Integer, ByVal nRegId As Integer)
            MyBase.AuthRegister(nRegId)
        End Sub

        Protected Overrides Sub OnLineRegisterTrying(ByVal sLineName As String)
            Dim sStatus As String = sLineName & " " & cDefine.LINE_REG_TRYING
            GenStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnLineRegisterFailed(ByVal sLineName As String, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)
            Dim sStatus As String = sLineName & " Register Failed: " & sReasonPhrase
            GenStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnLineRegisterSuccess(ByVal sLineName As String)
            Dim sStatus As String = sLineName & " " & cDefine.LINE_REG_SUCCESS
            GenStatusLog(sStatus)
        End Sub

        Public Sub GenStatusLog(ByVal sStatus As String)
            m_objMainForm.OnVaxStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnVaxErrorLog(ByVal sFuncName As String, ByVal nErrorCode As Integer, ByVal sErrorMsg As String)
            Dim sMsg As String = "[Method: " & sFuncName & "] [Error Msg: " & sErrorMsg & "] [Code: " & nErrorCode.ToString() & "]"
            GenStatusLog(sMsg)
        End Sub

        Protected Overrides Sub OnCallSessionErrorLog(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nErrorCode As Integer, ByVal sErrorMsg As String)
            Dim sMsg As String = "[Session: " & nSessionId.ToString() & "] [Error Msg: " & sErrorMsg & "] [Code: " & nErrorCode.ToString() & "]"
            GenStatusLog(sMsg)
        End Sub

        Protected Overrides Sub OnOutgoingDiagnosticLog(ByVal sMsgSIP As String, ByVal sToIP As String, ByVal nToPort As Integer)
            Dim LogFile As FileStream = New FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite)
            LogFile.Seek(0, SeekOrigin.[End])
            Dim sNewLine As String = vbCrLf
            Dim aNewLine As Byte() = Encoding.ASCII.GetBytes(sNewLine)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sAddressInfo As String = String.Format("{0}:{1}", sToIP, nToPort)
            Dim aAddressInfo As Byte() = Encoding.ASCII.GetBytes(sAddressInfo)
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sLogType As String = "============== OUTGOING============"
            Dim aLogType As Byte() = Encoding.ASCII.GetBytes(sLogType)
            LogFile.Write(aLogType, 0, sLogType.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim aMsgSIP As Byte() = Encoding.ASCII.GetBytes(sMsgSIP)
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length)
            LogFile.Close()
        End Sub

        Protected Overrides Sub OnIncomingDiagnosticLog(ByVal sMsgSIP As String, ByVal sFromIP As String, ByVal nFromPort As Integer)
            Dim LogFile As FileStream = New FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite)
            LogFile.Seek(0, SeekOrigin.[End])
            Dim sNewLine As String = vbCrLf
            Dim aNewLine As Byte() = Encoding.ASCII.GetBytes(sNewLine)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sAddressInfo As String = String.Format("{0}:{1}", sFromIP, nFromPort)
            Dim aAddressInfo As Byte() = Encoding.ASCII.GetBytes(sAddressInfo)
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sLogType As String = "============== INCOMING ============"
            Dim aLogType As Byte() = Encoding.ASCII.GetBytes(sLogType)
            LogFile.Write(aLogType, 0, sLogType.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim aMsgSIP As Byte() = Encoding.ASCII.GetBytes(sMsgSIP)
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length)
            LogFile.Close()
        End Sub
    End Class
End Namespace
