﻿Imports System.Text
Imports System.IO

Namespace BasicIVR
    Public Class cVaxServerExCOM
        Inherits cVaxServerCOM

        Public Shared mVaxExCOM As cVaxServerExCOM

        Private m_nPlayMusic As Integer = -1
        Private m_bInitialized As Boolean = False
        Private m_bIsLineAdded As Boolean = False

        Private m_bPlayWaveConnected As Boolean = True
        Private m_bDetectDTMF As Boolean = True
        Private m_objSessionList As cVaxCallSessionList = Nothing

        Private m_objMainForm As MainForm = Nothing
        Public Const VAX_ACCOUNT_LINE_SIP As String = "LineSIP"

        Public Sub New(ByVal objMainForm As MainForm)
            mVaxExCOM = Me
            m_objMainForm = objMainForm
        End Sub

        Protected Overrides Sub Finalize()
        End Sub

        Private Sub ResetAll()
            m_objSessionList = New cVaxCallSessionList()
        End Sub

        Public Overloads Function Initialize(ByVal nListenPort As Integer) As Boolean
            UnInitialize()
            MyBase.SetLicenseKey("TRIAL-LICENSE-KEY")
            If Not MyBase.Initialize("") Then Return False

            If Not MyBase.OpenNetworkUDP("", nListenPort) Then
                UnInitialize()
                Return False
            End If

            ' MyBase.AttackDetectScanSIP("demo.vaxvoip.com")
            ' MyBase.AttackDetectFloodSIP(1000)
            ' MyBase.AttackDetectBruteForceSIP(10, 2)

            MyBase.SetListenPortRangeRTP(20000, 40000)
            MyBase.AudioSessionLost(True, 20)
            MyBase.VideoCOMM(False)

            For nCount As Integer = 4000 To 4080
                MyBase.AddUser(nCount.ToString(), nCount.ToString(), "32")
            Next

            m_bInitialized = True
            Return True
        End Function

        Public Overloads Sub UnInitialize()
            m_bInitialized = False
            UnLoadMusic()
            ResetAll()
            m_nPlayMusic = -1
            m_bIsLineAdded = False
            MyBase.UnInitialize()
        End Sub

        Public Function IsInitialized() As Boolean
            Return m_bInitialized
        End Function

        Public Function RegisterAccountSIP(ByVal sDisplayName As String, ByVal sUserName As String, ByVal sAuthUser As String, ByVal sAuthPwd As String, ByVal sDomainRealm As String, ByVal sServerAddr As String, ByVal nServerPort As Integer) As Boolean
            If Not MyBase.AddLine(VAX_ACCOUNT_LINE_SIP, VAX_LINE_TYPE_UDP, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sServerAddr, nServerPort, "32") Then Return False
            If Not RegisterLine(VAX_ACCOUNT_LINE_SIP, 1800) Then Return False
            m_bIsLineAdded = True
            Return True
        End Function

        Public Sub UnRegisterAccountSIP()
            MyBase.UnRegisterLine(VAX_ACCOUNT_LINE_SIP)
            MyBase.RemoveLine(VAX_ACCOUNT_LINE_SIP)
            m_bIsLineAdded = False
        End Sub

        Public Function IsAddedAccountSIP() As Boolean
            Return m_bIsLineAdded
        End Function

        Public Sub CloseCall(ByVal nPhoneNoId As UInt32)
            Dim objCallSession As cVaxCallSession = m_objSessionList.FindPhoneNoSession(nPhoneNoId)
            If objCallSession Is Nothing Then Return
            CloseCallSession(objCallSession.m_nSessionId)
        End Sub

        Public Function LoadMusic(ByVal sWaveFile As String) As Boolean
            UnLoadMusic()
            If sWaveFile.Length = 0 Then Return False
            m_nPlayMusic = MyBase.LoadWaveFile(sWaveFile)
            If m_nPlayMusic = -1 Then Return False
            Return True
        End Function

        Public Sub UnLoadMusic()
            If m_nPlayMusic = -1 Then Return
            MyBase.UnLoadWaveID(m_nPlayMusic)
            m_nPlayMusic = -1
        End Sub

        Public Sub PlayMusic(ByVal nSessionId As Integer)
            If m_nPlayMusic = -1 Then Return
            MyBase.PlayWaveSetModeTypeToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_PLAY_WAVE_MODE_MIXED)
            MyBase.PlayWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, m_nPlayMusic, False, 10)
        End Sub

        Public Sub PlayText(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal sText As String)
            Dim aDataPCM As Byte() = Nothing
            Dim nDataSize As Integer = 0

            Dim objSpeech As cTextToSpeech = New cTextToSpeech()
            objSpeech.GenSpeech(sText, aDataPCM, nDataSize)

            MyBase.SendAudioPCM(nSessionId, nChannelId, aDataPCM, nDataSize)
        End Sub

        Public Sub EnableDetectDTMF(ByVal bDetectDTMF As Boolean)
            m_bDetectDTMF = bDetectDTMF
        End Sub

        Public Sub SetPlayWaveConnected(ByVal bPlayWaveConnected As Boolean)
            m_bPlayWaveConnected = bPlayWaveConnected
        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        Protected Overrides Sub OnCallSessionCreated(ByVal nSessionId As Integer, ByVal nReasonCode As Integer)
            m_objSessionList.AddSession(nSessionId)
        End Sub

        Protected Overrides Sub OnCallSessionClosed(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nReasonCode As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            If objCallSession.IsConnected() Then
                objCallSession.EndCall()
                Dim sDuration As String = objCallSession.GetDuration()
                m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, "", "", sDuration, "")
            End If

            objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode)
            m_objSessionList.RemoveSession(nSessionId)

        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        Protected Overrides Sub OnCallSessionAccessAudioPCM(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nAccessType As Integer, ByVal aDataPCM As Byte(), ByVal nSizePCM As Integer, ByVal nTypePCM As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, aDataPCM, nSizePCM)

        End Sub

        '//////////////////////////////////////////////////////////////////////////////////////////////////////
        '//////////////////////////////////////////////////////////////////////////////////////////////////////

        Protected Overrides Sub OnIncomingCall(ByVal nSessionId As Integer, ByVal sCallerName As String, ByVal sCallerId As String, ByVal sDialNo As String, ByVal nFromPeerType As Integer, ByVal sFromPeerName As String, ByVal sRouteUID As String, ByVal nRouteType As Integer, ByVal sRouteName As String, ByVal sUserAgentName As String, ByVal sFromIP As String, ByVal nFromPort As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            Dim sPhoneNo As String = sCallerName

            If sCallerName.Length = 0 Then

                If sCallerId.Length <> 0 Then
                    sPhoneNo = sCallerId
                Else
                    sPhoneNo = "Unknown"
                End If
            End If

            Dim nPhoneNoId As UInteger = m_objMainForm.InsertListItem(sPhoneNo, cDefine.PHONE_NO_PROCESSING, "", "", "")

            objCallSession.m_sPhoneNo = sPhoneNo
            objCallSession.m_sStatus = cDefine.PHONE_NO_PROCESSING
            objCallSession.m_nPhoneNoId = nPhoneNoId

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_PROCESSING, "", "", "")
            MyBase.AcceptCallSession(nSessionId, "", "", "", "", "", 20)

        End Sub

        Protected Overrides Sub OnCallSessionConnected(ByVal nSessionId As Integer)

            If m_bPlayWaveConnected Then PlayMusic(nSessionId)

            Dim objGenFileLoc As CGenFileLocation = New CGenFileLocation()
            Dim sRecFilePath As String = objGenFileLoc.GenRecordingFilePath(nSessionId, "RecordFiles")

            RecordWaveStartToCallSession(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, sRecFilePath, cVaxServerCOM.VAX_WAVE_REC_TYPE_MONO_PCM)

            If m_bDetectDTMF Then
                MyBase.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_DTMF_TYPE_RFC2833, True, 1500)
                MyBase.DetectDigitDTMF(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO, cVaxServerCOM.VAX_DTMF_TYPE_SIP_INFO, True, 1500)
            End If


            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionConnected(nSessionId, cVaxServerCOM.VAX_CALL_CHANNEL_ZERO)

            objCallSession.StartCall()
            objCallSession.SetConnected()
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_CONNECTED, "", "", sRecFilePath)
        End Sub

        Protected Overrides Sub OnDetectedDigitDTMF(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal sDigitDTMF As String, ByVal nTypeDTMF As Integer)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, "", sDigitDTMF, "", "")

            Dim sMsgDTMF As String = "Digit Detected: [ " & sDigitDTMF & " ]"
            PlayText(nSessionId, nChannelId, sMsgDTMF)

            GenStatusLog(sMsgDTMF)
        End Sub

        Protected Overrides Sub OnCallSessionFailed(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String, ByVal sContact As String)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            Dim sStatus As String
            sStatus = cDefine.PHONE_NO_FAILED & " : " & nStatusCode.ToString() & "-" & sReasonPhrase
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, sStatus, "", "", "")
        End Sub

        Protected Overrides Sub OnCallSessionTimeout(ByVal nSessionId As Integer, ByVal nChannelId As Integer)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_TIMEOUT, "", "", "")
        End Sub

        Protected Overrides Sub OnCallSessionCancelled(ByVal nSessionId As Integer)
            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return
            m_objMainForm.OnPhoneNoStatusLog(objCallSession.m_nPhoneNoId, objCallSession.m_sPhoneNo, cDefine.PHONE_NO_CANCELED, "", "", "")
        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        Protected Overrides Sub OnCallSessionMediaDetectStartedVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectStartedVAD(nSessionId, nChannelId)

        End Sub

        Protected Overrides Sub OnCallSessionMediaDetectCompletedVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal aDataPCM As Byte(), ByVal nSizePCM As Integer, ByVal nTypePCM As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectCompletedVAD(nSessionId, nChannelId, aDataPCM, nSizePCM, nTypePCM)

        End Sub

        Protected Overrides Sub OnCallSessionMediaDetectExpiredVAD(ByVal nSessionId As Integer, ByVal nChannelId As Integer)

            Dim objCallSession As cVaxCallSession = m_objSessionList.GetSession(nSessionId)
            If objCallSession Is Nothing Then Return

            objCallSession.OnCallSessionMediaDetectExpiredVAD(nSessionId, nChannelId)

        End Sub

        '////////////////////////////////////////////////////////////////////////////////////////////////////////
        '////////////////////////////////////////////////////////////////////////////////////////////////////////

        Protected Overrides Sub OnRegisterUser(ByVal sUserLogin As String, ByVal sDomain As String, ByVal sUserAgentName As String, ByVal sFromIP As String, ByVal nFromPort As Integer, ByVal nRegId As Integer)
            MyBase.AuthRegister(nRegId)
        End Sub

        Protected Overrides Sub OnLineRegisterTrying(ByVal sLineName As String)
            Dim sStatus As String = sLineName & " " & cDefine.LINE_REG_TRYING
            GenStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnLineRegisterFailed(ByVal sLineName As String, ByVal nStatusCode As Integer, ByVal sReasonPhrase As String)
            Dim sStatus As String = sLineName & " Register Failed: " & sReasonPhrase
            GenStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnLineRegisterSuccess(ByVal sLineName As String)
            Dim sStatus As String = sLineName & " " & cDefine.LINE_REG_SUCCESS
            GenStatusLog(sStatus)
        End Sub

        Public Sub GenStatusLog(ByVal sStatus As String)
            m_objMainForm.OnVaxStatusLog(sStatus)
        End Sub

        Protected Overrides Sub OnVaxErrorLog(ByVal sFuncName As String, ByVal nErrorCode As Integer, ByVal sErrorMsg As String)
            Dim sMsg As String = "[Method: " & sFuncName & "] [Error Msg: " & sErrorMsg & "] [Code: " & nErrorCode.ToString() & "]"
            GenStatusLog(sMsg)
        End Sub

        Protected Overrides Sub OnCallSessionErrorLog(ByVal nSessionId As Integer, ByVal nChannelId As Integer, ByVal nErrorCode As Integer, ByVal sErrorMsg As String)
            Dim sMsg As String = "[Session: " & nSessionId.ToString() & "] [Error Msg: " & sErrorMsg & "] [Code: " & nErrorCode.ToString() & "]"
            GenStatusLog(sMsg)
        End Sub

        Protected Overrides Sub OnOutgoingDiagnosticLog(ByVal sMsgSIP As String, ByVal sToIP As String, ByVal nToPort As Integer)
            Dim LogFile As FileStream = New FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite)
            LogFile.Seek(0, SeekOrigin.[End])
            Dim sNewLine As String = vbCrLf
            Dim aNewLine As Byte() = Encoding.ASCII.GetBytes(sNewLine)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sAddressInfo As String = String.Format("{0}:{1}", sToIP, nToPort)
            Dim aAddressInfo As Byte() = Encoding.ASCII.GetBytes(sAddressInfo)
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sLogType As String = "============== OUTGOING============"
            Dim aLogType As Byte() = Encoding.ASCII.GetBytes(sLogType)
            LogFile.Write(aLogType, 0, sLogType.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim aMsgSIP As Byte() = Encoding.ASCII.GetBytes(sMsgSIP)
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length)
            LogFile.Close()
        End Sub

        Protected Overrides Sub OnIncomingDiagnosticLog(ByVal sMsgSIP As String, ByVal sFromIP As String, ByVal nFromPort As Integer)
            Dim LogFile As FileStream = New FileStream("VaxLog.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite)
            LogFile.Seek(0, SeekOrigin.[End])
            Dim sNewLine As String = vbCrLf
            Dim aNewLine As Byte() = Encoding.ASCII.GetBytes(sNewLine)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sAddressInfo As String = String.Format("{0}:{1}", sFromIP, nFromPort)
            Dim aAddressInfo As Byte() = Encoding.ASCII.GetBytes(sAddressInfo)
            LogFile.Write(aAddressInfo, 0, sAddressInfo.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim sLogType As String = "============== INCOMING ============"
            Dim aLogType As Byte() = Encoding.ASCII.GetBytes(sLogType)
            LogFile.Write(aLogType, 0, sLogType.Length)
            LogFile.Write(aNewLine, 0, sNewLine.Length)
            Dim aMsgSIP As Byte() = Encoding.ASCII.GetBytes(sMsgSIP)
            LogFile.Write(aMsgSIP, 0, sMsgSIP.Length)
            LogFile.Close()
        End Sub
    End Class
End Namespace
